# -*- coding: utf-8 -*-
# @Time    : 24/01/2023
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : ------------
# @Software: PyCharm

#import opendssdirect as drt
import py_dss_interface

dss = py_dss_interface.DSSDLL()


# -*- coding: utf-8 -*-
# @Time    : 30/12/2022
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : ------------
# @Software: PyCharm
import pandas as pd
from .error_handling_logging import orient, indent

class performance_metrics:
    def __init__(self, V_Bus: pd.DataFrame, I_Elem: pd.DataFrame):
        self.V_Bus = V_Bus
        self.I_Elem = I_Elem

    def Mean_Absolute_Error(self):
        dict_MAE = dict()
        if not self.V_Bus.empty:
            n = len(self.V_Bus)
            sum_mod = 0
            sum_ang = 0
            for index, col in self.V_Bus.iterrows():
                sum_mod = abs(self.V_Bus['V1(pu)_DSS'][index] - self.V_Bus['V1(pu)_EST'][index])
                sum_ang = abs(self.V_Bus['Ang1(deg)_DSS'][index] - self.V_Bus['Ang1(deg)_EST'][index])
                sum_mod += sum_mod
                sum_ang += sum_ang
            dict_MAE['V1(pu)'] = format(sum_mod / n, '.2E')
            dict_MAE['AngV1(deg)'] = format(sum_ang / n, '.2E')

        if not self.I_Elem.empty:
            n = len(self.I_Elem)
            sum_mod = 0
            sum_ang = 0
            for index, col in self.I_Elem.iterrows():
                sum_mod = abs(self.I_Elem['I1(pu)_DSS'][index] - self.I_Elem['I1(pu)_EST'][index])
                sum_ang = abs(self.I_Elem['Ang1(deg)_DSS'][index] - self.I_Elem['Ang1(deg)_EST'][index])

                sum_mod += sum_mod
                sum_ang += sum_ang

            dict_MAE['I1(pu)'] = format(sum_mod / n, '.2E')
            dict_MAE['AngI1(deg)'] = format(sum_ang / n, '.2E')

        df_MAE = pd.DataFrame([[key, dict_MAE[key]] for key in dict_MAE.keys()], columns=['Variable', 'MAE'])
        return df_MAE

    def Mean_Absolute_Percentage_Error(self):
        dict_MAPE = dict()
        if not self.V_Bus.empty:
            n = len(self.V_Bus)
            sum_mod = 0
            sum_ang = 0
            for index, col in self.V_Bus.iterrows():
                sum_mod = abs(
                    self.V_Bus['V1(pu)_EST'][index] - self.V_Bus['V1(pu)_DSS'][index]
                ) / self.V_Bus['V1(pu)_DSS'][index]
                sum_ang = abs(
                    self.V_Bus['Ang1(deg)_EST'][index] - self.V_Bus['Ang1(deg)_DSS'][index]
                ) / self.V_Bus['Ang1(deg)_DSS'][index]
                sum_mod += sum_mod
                sum_ang += sum_ang
            dict_MAPE['V1(pu)'] = format(sum_mod / n, '.2E')
            dict_MAPE['AngV1(deg)'] = format(sum_ang / n, '.2E')

        if not self.I_Elem.empty:
            n = len(self.I_Elem)
            sum_mod = 0
            sum_ang = 0
            for index, col in self.I_Elem.iterrows():
                sum_mod = abs(
                    self.I_Elem['I1(pu)_EST'][index] - self.I_Elem['I1(pu)_DSS'][index]
                ) / self.I_Elem['I1(pu)_DSS'][index]
                sum_ang = abs(
                    self.I_Elem['Ang1(deg)_EST'][index] - self.I_Elem['Ang1(deg)_DSS'][index]
                ) / self.I_Elem['Ang1(deg)_DSS'][index]
                sum_mod += sum_mod
                sum_ang += sum_ang

            dict_MAPE['I1(pu)'] = format(sum_mod / n, '.2E')
            dict_MAPE['AngI1(deg)'] = format(sum_ang / n, '.2E')

        df_MAPE = pd.DataFrame([[key, dict_MAPE[key]] for key in dict_MAPE.keys()], columns=['Variable', 'MAPE'])

        return df_MAPE

    def Root_Mean_Square_Error(self):
        dict_RMSE = dict()
        if not self.V_Bus.empty:
            n = len(self.V_Bus)
            sum_mod = 0
            sum_ang = 0
            for index, col in self.V_Bus.iterrows():
                sum_mod = abs(
                    self.V_Bus['V1(pu)_EST'][index] - self.V_Bus['V1(pu)_DSS'][index]
                ) ** 2
                sum_ang = abs(
                    self.V_Bus['Ang1(deg)_EST'][index] - self.V_Bus['Ang1(deg)_DSS'][index]
                ) ** 2
                sum_mod += sum_mod
                sum_ang += sum_ang

            dict_RMSE['V1(pu)'] = format((sum_mod / n) ** 0.5, '.2E')
            dict_RMSE['AngV1(deg)'] = format((sum_ang / n) ** 0.5, '.2E')

        if not self.I_Elem.empty:
            n = len(self.I_Elem)
            sum_mod = 0
            sum_ang = 0
            for index, col in self.I_Elem.iterrows():
                sum_mod = abs(
                    self.I_Elem['I1(pu)_EST'][index] - self.I_Elem['I1(pu)_DSS'][index]
                ) ** 2
                sum_ang = abs(
                    self.I_Elem['Ang1(deg)_EST'][index] - self.I_Elem['Ang1(deg)_DSS'][index]
                ) ** 2
                sum_mod += sum_mod
                sum_ang += sum_ang


            dict_RMSE['I1(pu)'] = format((sum_mod / n) ** 0.5, '.2E')
            dict_RMSE['AngI1(deg)'] = format((sum_ang / n) ** 0.5, '.2E')

        df_RMSE = pd.DataFrame([[key, dict_RMSE[key]] for key in dict_RMSE.keys()], columns=['Variable', 'RMSE'])

        return df_RMSE

    def _dict_merge(
            self,
            MAE: bool,
            MAPE: bool,
            RMSE: bool,
            perf_dict: dict,
            View_res: bool,
            path_save: str,
            name_project: str
    ):
        if MAE and MAPE and RMSE:
            df_dict =perf_dict['MAE'].merge(perf_dict['MAPE'], on='Variable').merge(perf_dict['RMSE'], on='Variable')
        elif MAE and MAPE:
            df_dict = perf_dict['MAE'].merge(perf_dict['MAPE'], on='Variable')
        elif MAE and RMSE:
            df_dict = perf_dict['MAE'].merge(perf_dict['RMSE'], on='Variable')
        elif MAPE and RMSE:
            df_dict = perf_dict['MAPE'].merge(perf_dict['RMSE'], on='Variable')
        elif MAE:
            df_dict = perf_dict['MAE']
        elif MAPE:
            df_dict = perf_dict['MAPE']
        elif RMSE:
            df_dict = perf_dict['RMSE']

        if View_res:
            print(df_dict)
        if path_save is not None:
            df_dict.to_excel(f'{path_save}\Metrics_{name_project}.xlsx', index=False)
            df_dict_json = df_dict.to_json(orient=orient, indent=indent)
            with open(f"{path_save}\Metrics_{name_project}.json", "w") as outfile:
                outfile.write(df_dict_json)
        return df_dict


# -*- coding: utf-8 -*-
# @Time    : 18/08/2022
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : ------------
# @Software: PyCharm

import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd
from .OpenDSS_data_extraction import *

class _Plot_results_DSS_EST:
    def __init__(
            self,
            name_project: str,
            path_save: str,
            no_PU: bool,
            View: bool
    ):
        self.no_PU = no_PU
        self.path_save = path_save
        self.name_project = name_project
        self.View = View

    def _Vi_Angi_1ph_Pos(
            self,
            df_DSS_EST: pd.DataFrame
    ):
        df_DSS_EST = df_DSS_EST.rename(columns={'bus_name': 'Bus name'})
        if self.no_PU:
            df_Vi_no_pu = df_DSS_EST.copy()
            for index, row in df_DSS_EST.iterrows():
                dss.circuit_set_active_bus(df_DSS_EST['Bus name'][index])
                kVbase = dss.bus_kv_base()
                df_Vi_no_pu['V1(kV)_EST'][index] = df_DSS_EST['V1(pu)_EST'][index] * kVbase
                df_Vi_no_pu['V1(kV)_DSS'][index] = df_DSS_EST['V1(pu)_DSS'][index] * kVbase
            fig, axes = plt.subplots(nrows=1, ncols=2, figsize=(12, 7))
            df1 = pd.melt(
                df_DSS_EST,
                id_vars=['Bus name'],
                value_vars=['V1(kV)_DSS', 'V1(kV)_EST'])
            df1['Bus name'] = df1['Bus name'].astype(str)
            sns.lineplot(
                x="Bus name",
                hue="variable",
                y="value",
                data=df1,
                ax=axes[0],
                palette='deep')
            axes[0].set_ylabel('kV')
            axes[0].legend(loc='best')
            axes[0].tick_params(axis='x', rotation=90)
            axes[0].grid(axis='y', linestyle='--', linewidth=0.5)

            df2 = pd.melt(
                df_DSS_EST,
                id_vars=['Bus name'],
                value_vars=['Ang1(deg)_DSS', 'Ang1(deg)_EST'])
            df2['Bus name'] = df2['Bus name'].astype(str)
            sns.lineplot(
                x="Bus name",
                hue="variable",
                y="value",
                data=df2,
                ax=axes[1],
                palette='deep')
            axes[1].set_ylabel('deg')
            axes[1].legend(loc='best')
            axes[1].tick_params(axis='x', rotation=90)
            axes[1].grid(axis='y', linestyle='--', linewidth=0.5)
            if self.path_save is not None:
                plt.savefig(f'{self.path_save}./Vi_DSS_EST_PU_{self.name_project}.png')
            if self.View:
                plt.show()
        else:
            fig, axes = plt.subplots(nrows=1, ncols=2, figsize=(12, 7))
            df1 = pd.melt(
                df_DSS_EST,
                id_vars=['Bus name'],
                value_vars=['V1(pu)_DSS', 'V1(pu)_EST'])
            df1['Bus name'] = df1['Bus name'].astype(str)
            sns.lineplot(
                x="Bus name",
                hue="variable",
                y="value",
                data=df1,
                ax=axes[0],
                palette='deep')
            axes[0].set_ylabel('p.u.')
            axes[0].legend(loc='best')
            axes[0].tick_params(axis='x', rotation=90)
            axes[0].grid(axis='y', linestyle='--', linewidth=0.5)

            df2 = pd.melt(
                df_DSS_EST,
                id_vars=['Bus name'],
                value_vars=['Ang1(deg)_DSS', 'Ang1(deg)_EST'])
            df2['Bus name'] = df2['Bus name'].astype(str)
            sns.lineplot(
                x="Bus name",
                hue="variable",
                y="value",
                data=df2,
                ax=axes[1],
                palette='deep')
            axes[1].set_ylabel('deg')
            axes[1].legend(loc='best')
            axes[1].tick_params(axis='x', rotation=90)
            axes[1].grid(axis='y', linestyle='--', linewidth=0.5)
            if self.path_save is not None:
                plt.savefig(f'{self.path_save}./Vi_DSS_EST_PU_{self.name_project}.png')
            if self.View:
                plt.show()
    def _Iij_Angij_1ph_Pos(
            self,
            df_DSS_EST: pd.DataFrame,
            SbasMVA_3ph: float = None):
        df_DSS_EST = df_DSS_EST.rename(
            columns={'element_name': 'Element name'}
        )
        df_DSS_EST['Element name'] = df_DSS_EST['Element name'].replace({'Line.': ''}, regex=True)
        if self.no_PU == True:
            I_ang_no_pu = df_DSS_EST.copy()
            I_ang_no_pu = I_ang_no_pu.rename(
                columns={'I1(pu)_EST': 'I1(A)_EST', 'I1(pu)_DSS': 'I1(A)_DSS'}
            )
            for index, row in I_ang_no_pu.iterrows():
                dss.circuit_set_active_bus(I_ang_no_pu['from_bus'][index])
                kVbase = dss.bus_kv_base()
                Ibas = ((SbasMVA_3ph / 3) * 1000) / kVbase
                I_ang_no_pu['I1(A)_EST'][index] = I_ang_no_pu['I1(A)_EST'][index] * Ibas
                I_ang_no_pu['I1(A)_DSS'][index] = I_ang_no_pu['I1(A)_DSS'][index] * Ibas
            fig, axes = plt.subplots(nrows=1, ncols=2, figsize=(12, 7))
            df1 = pd.melt(
                df_DSS_EST,
                id_vars=['Element name'],
                value_vars=['I1(A)_DSS', 'I1(A)_EST'])
            df1['Element name'] = df1['Element name'].astype(str)
            sns.lineplot(
                x="Bus name",
                hue="variable",
                y="value",
                data=df1,
                ax=axes[0],
                palette='deep')
            axes[0].set_ylabel('A')
            axes[0].legend(loc='best')
            axes[0].tick_params(axis='x', rotation=90)
            axes[0].grid(axis='y', linestyle='--', linewidth=0.5)
            df2 = pd.melt(
                df_DSS_EST,
                id_vars=['Element name'],
                value_vars=['Ang1(deg)_DSS', 'Ang1(deg)_EST'])
            df2['Element name'] = df2['Element name'].astype(str)
            sns.lineplot(
                x="Bus name",
                hue="variable",
                y="value",
                data=df2,
                ax=axes[1],
                palette='deep')
            axes[1].set_ylabel('deg')
            axes[1].legend(loc='best')
            axes[1].tick_params(axis='x', rotation=90)
            axes[1].grid(axis='y', linestyle='--', linewidth=0.5)
            if self.path_save is not None:
                plt.savefig(f'{self.path_save}./Iij_DSS_EST_no_PU_{self.name_project}.png')
            if self.View:
                plt.show()
        fig, axes = plt.subplots(nrows=1, ncols=2, figsize=(12, 7))
        df1 = pd.melt(
            df_DSS_EST,
            id_vars=['Element name'],
            value_vars=['I1(pu)_DSS', 'I1(pu)_EST'])
        df1['Element name'] = df1['Element name'].astype(str)
        sns.lineplot(
            x="Element name",
            hue="variable",
            y="value",
            data=df1,
            ax=axes[0],
            palette='deep')
        axes[0].set_ylabel('p.u.')
        axes[0].legend(loc='best')
        axes[0].tick_params(axis='x', rotation=90)
        axes[0].grid(axis='y', linestyle='--', linewidth=0.5)
        df2 = pd.melt(
            df_DSS_EST,
            id_vars=['Element name'],
            value_vars=['Ang1(deg)_DSS', 'Ang1(deg)_EST'])
        df2['Element name'] = df2['Element name'].astype(str)
        sns.lineplot(
            x="Element name",
            hue="variable",
            y="value",
            data=df2,
            ax=axes[1],
            palette='deep')
        axes[1].set_ylabel('deg')
        axes[1].legend(loc='best')
        axes[1].tick_params(axis='x', rotation=90)
        axes[1].grid(axis='y', linestyle='--', linewidth=0.5)
        if self.path_save is not None:
            plt.savefig(f'{self.path_save}./Iij_DSS_EST_no_PU_{self.name_project}.png')
        if self.View:
            plt.show()


# -*- coding: utf-8 -*-
# @Time    : 18/08/2022
# @Author  : Ing. Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : ------------
# @Software: PyCharm

import pandas as pd
import numpy as np
import math
import cmath

from openpy_dsse import YBus_Matrix_Pos_Seq

from .OpenDSS_data_extraction import OpenDSS_data_collection, Values_per_unit, dss
from openpy_dsse.DSSE_algorithms.Symb_Eqn.sym_func_1ph import sym_func_Iij_rec
from .YBus_Matrix_Pos_Seq import get_G_B_Pos_seq, YBus_Matrix_SeqPos_OpenDSS
from .error_handling_logging import elem_YBus, orient, indent

data_DSS = OpenDSS_data_collection()

class estimated_electrical_variables:
    def __init__(self, I_Ang_EST: bool = False, PQi_EST: bool = False, PQf_EST: bool = False):
        self.I_Ang_EST = I_Ang_EST
        self.PQi_EST = PQi_EST
        self.PQf_EST = PQf_EST
    def Imang_Angle_EST_1ph(
            self,
            df_Results_DSSE: pd.DataFrame,
            SbasMVA_3ph: float,
            path_save: str,
            View_res: bool,
            DSS_col: bool,
            name_project: str
    ):
        I_Ang_EST_DSS = dict()
        YBus_1ph = YBus_Matrix_SeqPos_OpenDSS(data_DSS.allbusnames_aux())
        YBusMatrix_PU = YBus_1ph.YBus_Matrix_pu(
            SbasMVA_3ph=SbasMVA_3ph,
            YBus_Matrix=YBus_1ph.build_YBus_Matrix_Pos_Seq(
                element=elem_YBus
            )
        )
        Per_PU = Values_per_unit(SbasMVA_3ph=SbasMVA_3ph)
        I_elem_pu_DSS = Per_PU.element_Iij_PU(
            df_element_currents=data_DSS.element_currents_Iij_Ang(
                element=['lines'])
        )
        #Current
        I_elem_pu_DSS = I_elem_pu_DSS[['element_name', 'from_bus', 'to_bus', 'I1(pu)', 'ang1(deg)']]
        DF_Iij_aux_DSS = pd.merge(
            I_elem_pu_DSS, data_DSS.allbusnames_aux(),
            how='inner', left_on='from_bus', right_on='bus_name', suffixes=('_i', '')
        )
        DF_Iij_aux_DSS = pd.merge(
            DF_Iij_aux_DSS, data_DSS.allbusnames_aux(),
            how='inner', left_on='to_bus', right_on='bus_name', suffixes=('_j', '')
        )
        DF_Iij_aux_DSS = DF_Iij_aux_DSS.rename(
            columns={'bus_name_aux_j': 'from_bus_aux', 'bus_name_aux': 'to_bus_aux'}
        )
        DF_I_Ang_estimate = pd.DataFrame(
            columns=['element_name', 'from_bus', 'to_bus', 'I1(pu)', 'ang1(deg)'])
        DF_results_WLS = df_Results_DSSE.copy()
        for k in range(len(DF_results_WLS)):
            'angle'
            DF_results_WLS.at[k, 'Ang1(deg)_EST'] = math.radians(DF_results_WLS.at[k, 'Ang1(deg)_EST'])
        Matrix_WLS_results = np.array(DF_results_WLS)
        V_1 = np.array([Matrix_WLS_results[:, 2]]).T
        Vang_1 = np.array([Matrix_WLS_results[:, 3]]).T

        f29 = sym_func_Iij_rec()
        for index, row in DF_Iij_aux_DSS.iterrows():
            id_name = DF_Iij_aux_DSS['element_name'][index]
            m = DF_Iij_aux_DSS['from_bus_aux'][index] - 1
            n = DF_Iij_aux_DSS['to_bus_aux'][index] - 1

            G1, B1 = get_G_B_Pos_seq(m, n, YBusMatrix_PU)
            I_ij = f29(G1, B1, V_1[m][0], Vang_1[m][0], V_1[n][0], Vang_1[n][0])
            I1, ang1 = cmath.polar(I_ij)
            DF_I_Ang_estimate = DF_I_Ang_estimate.append({
                'element_name': id_name,
                'from_bus': DF_Iij_aux_DSS['from_bus'][index],
                'to_bus': DF_Iij_aux_DSS['to_bus'][index],
                'I1(pu)': I1, 'ang1(deg)': math.degrees(ang1) - 180}, ignore_index=True)

        I_Ang_EST_DSS['df_EST'] = DF_I_Ang_estimate.rename(columns={'I1(pu)': 'I1(pu)_EST', 'ang1(deg)': 'Ang1(deg)_EST'})
        I_Ang_EST_DSS['df_DSS'] = I_elem_pu_DSS.rename(columns={'I1(pu)': 'I1(pu)_DSS', 'ang1(deg)': 'Ang1(deg)_DSS'})
        if DSS_col:
            df_DSS_EST = pd.merge(I_Ang_EST_DSS['df_DSS'], I_Ang_EST_DSS['df_EST'], on=('element_name', 'from_bus', 'to_bus'))
        else:
            df_DSS_EST = I_Ang_EST_DSS['df_EST']
        if View_res:
            print(df_DSS_EST)
        if path_save is not None:
            df_DSS_EST.to_excel(f'{path_save}\Results_I_Ang_from_DSSE_{name_project}.xlsx', index=False)

            df_DSS_EST_json = df_DSS_EST.to_json(orient=orient, indent=indent)
            with open(f"{path_save}\Results_I_Ang_from_DSSE_{name_project}.json", "w") as outfile:
                outfile.write(df_DSS_EST_json)
        return I_Ang_EST_DSS

